"""Unit test for EAN-13 barcode encoder"""

__revision__ = "$Rev: 2025 $"

import unittest

from __init__ import EAN13Encoder

class EAN13Test(unittest.TestCase):
    """Unit test class for EAN13 bar code encoder"""

    test_strings = ("012345678901",
                    "007567816412",
                    "750103131130" )

    def test_check_digit(self):
        """Make sure the check digit calculation works"""

        # test includes the full range of check digits
        check_digits = {"012345678901": 2,
                        "007567816412": 5,
                        "750103131130": 9,
                        "000000000000": 0,
                        "000000010101": 1,
                        "000000001111": 2,
                        "000000000111": 3,
                        "000000000101": 4,
                        "000000001011": 5,
                        "000000001001": 6,
                        "000000000001": 7,
                        "000000001010": 8,
                        "000000000010": 9 } 
        for code, check in check_digits.items():
            enc = EAN13Encoder( code )
            self.assertEqual( enc.check_digit, check )

    def test_parity(self): 
        """Test the parity calculations"""
        enc = EAN13Encoder( "750103131130" )
        self.assertEqual( enc.get_parity( ), (1, 0, 1, 0, 1, 0) )


    def test_encoding(self):
        """Make the the left and right encodings work"""
        enc = EAN13Encoder( "750103131130" )
        left, right = enc.encode( )
        self.assertEqual( left, "011000101001110011001010011101111010110011" )
        self.assertEqual( right, "100001011001101100110100001011100101110100" )

    def test_against_generated(self):
        """Compare the output of this library with generated barcodes"""

        for index, string in enumerate(EAN13Test.test_strings):
            encoder = EAN13Encoder(string)
            encoder.save('test.png')

            import filecmp
            self.failUnless(filecmp.cmp('test.png',
                            'ean13/test_img/%d.png' % (index + 1)))
         

if __name__ == '__main__':
    unittest.main()



"""Code-128 barcode encoder

All needed by the user is done via the Code128Encoder class:

>>> encoder = Code128Encoder( "HuDoRa" )
>>> encoder.save( "test.png" )

Implemented by Helen Taylor for HUDORA GmbH.

Detailed documentation on the format here: 
http://www.barcodeisland.com/code128.phtml
http://www.adams1.com/pub/russadam/128code.html

You may use this under a BSD License.
"""

__revision__ = "$Rev: 1$"

from textencoder import TextEncoder
from renderer import Code128Renderer
import logging

log = logging.getLogger( "code128" )


class Code128Encoder:
    """Top-level class which handles the overall process of 
    encoding input string and outputting the result"""

    
    def __init__( self, text ):

        self.text = text
        encoder = TextEncoder( )

        self.encoded_text = encoder.encode( self.text )
        log.debug( "Encoded text is %s", self.encoded_text )

        self.checksum = self.calculate_check_sum( )
        log.debug( "Checksum is %d", self.checksum )

        self.bars = encoder.get_bars( self.encoded_text, self.checksum )
        log.debug( "Bars: %s", self.bars )


    def calculate_check_sum( self ):
        """Calculate the check sum of the encoded text.
        Checksum is based on the input text and the starting code,
        and a mod103 algorithm is used"""

        checksum = self.encoded_text[0]

        for index, char in enumerate( self.encoded_text ):
            if index > 0:
                checksum += (index * char)
            
        return checksum % 103


    def save( self, filename, bar_width=3 ):
        """Write the barcode out to an image file"""
        Code128Renderer( self.bars, self.text ).write_file( filename, bar_width )



import cPickle

frame_data = cPickle.loads("""(lp1
(lp2
I1
aI1
aI1
aI1
aI1
aI1
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI1
aI1
aI1
aI1
aI1
aI1
aa(lp3
I1
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI1
aa(lp4
I1
aI0
aI1
aI1
aI1
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI1
aI1
aI1
aI0
aI1
aa(lp5
I1
aI0
aI1
aI1
aI1
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI1
aI1
aI1
aI0
aI1
aa(lp6
I1
aI0
aI1
aI1
aI1
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI1
aI1
aI1
aI0
aI1
aa(lp7
I1
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI1
aa(lp8
I1
aI1
aI1
aI1
aI1
aI1
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI1
aI1
aI1
aI1
aI1
aI1
aa(lp9
I0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp10
I0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp11
I0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp12
I0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp13
I0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp14
I0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp15
I0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp16
I0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp17
I0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp18
I0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp19
I0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp20
I0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp21
I0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp22
I0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp23
I0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp24
I0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp25
I0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp26
I0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp27
I0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp28
I0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp29
I0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp30
I0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI1
aI1
aI1
aI1
aI0
aI0
aI0
aI0
aa(lp31
I0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aa(lp32
I1
aI1
aI1
aI1
aI1
aI1
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI1
aI0
aI1
aI0
aI0
aI0
aI0
aa(lp33
I1
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aa(lp34
I1
aI0
aI1
aI1
aI1
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI1
aI1
aI1
aI1
aI0
aI0
aI0
aI0
aa(lp35
I1
aI0
aI1
aI1
aI1
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp36
I1
aI0
aI1
aI1
aI1
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp37
I1
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp38
I1
aI1
aI1
aI1
aI1
aI1
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp39
a.""")



"""Generate test images for code128 barcode encoder"""

__revision__ = "$Rev: 1$"

from __init__ import Code128Encoder
from code128test import Code128Test

for index, string in enumerate(Code128Test.test_strings):
    enc = Code128Encoder(string)
    enc.save("code128/test_img/%d.png" % (index+1) )



"""QR Code renderer"""

__revision__ = "$Rev: 1927 $"

from cStringIO import StringIO 
import Image

class QRCodeRenderer:
    """Rendering class - given a pre-populated QR Code matrix.
    it will add edge handles and render to either to an image
    (including quiet zone) or ascii printout"""

    def __init__(self, matrix):

        self.mtx_size = len(matrix)
        self.matrix = matrix
    # end def __init__


    def add_border(self, colour=1, width=4):
        """Wrap the matrix in a border of given width
            and colour"""

        self.mtx_size += width * 2

        self.matrix = \
            [[colour, ] * self.mtx_size, ] * width + \
            [[colour, ] * width + self.matrix[i] + [colour, ] * width
        for i in range(0, self.mtx_size - (width * 2))] + \
            [[colour, ] * self.mtx_size, ] * width
        # end for
    # end def add_border



    def get_pilimage(self, cellsize):
        """Return the matrix as an PIL object""" 

        # add the quiet zone (4 x cell width)
        self.add_border(colour=0, width=4)

        # get the matrix into the right buffer format
        buff = self.get_buffer(cellsize)

        # write the buffer out to an image
        img = Image.frombuffer(
                    'L',
                    (self.mtx_size * cellsize, self.mtx_size * cellsize),
                    buff)
        return img

    def write_file( self, cellsize, filename ): 
        """Write the matrix out to an image file""" 
        img = self.get_pilimage( cellsize ) 
        img.save( filename ) 

    def get_imagedata( self, cellsize): 
        """Write the matrix out as PNG to an bytestream""" 
        buffer = StringIO() 
        img = self.get_pilimage( cellsize ) 
        img.save( buffer, "PNG" ) 
        return buffer.getvalue() 

    def get_buffer(self, cellsize):
        """Convert the matrix into the buffer format used by PIL"""

        def pixel(value):
            """return pixel representation of a matrix value
            0 => white, 1 => black"""
            if value == 0:
                return chr(255)
            elif value == 1:
                return chr(0)
            # end if
        # end def pixel

        # PIL writes image buffers from the bottom up,
        # so feed in the rows in reverse
        buf = ""
        for row in self.matrix[::-1]:
            bufrow = ''.join([pixel(cell) * cellsize for cell in row])
            for _ in range (0, cellsize):
                buf += bufrow
            # end for
        # end for
        return buf
    # end def get_buffer


    def get_ascii(self):
        """Write an ascii version of the matrix out to screen"""

        def symbol(value):
            """return ascii representation of matrix value"""
            if value == 0:
                return ' '
            elif value == 1:
                return 'X'
            # end if
        # end def symbol

        return '\n'.join([\
                ''.join([symbol(cell) for cell in row]) \
               for row in self.matrix]) + '\n'
    # end def get_ascii
# end class QRCodeRenderer




"""EAN-13 barcode encoder

All needed by the user is done via the EAN13Encoder class:

>>> encoder = EAN13Encoder( "012345678901" )
>>> encoder.save( "test.png" )

Implemented by Helen Taylor for HUDORA GmbH.

Detailed documentation on the format here: 
http://www.barcodeisland.com/ean13.phtml

You may use this under a BSD License.
"""

__revision__ = "$Rev: 1$"

import encoding
from renderer import EAN13Renderer

GUARDS = ("101", "01010", "101")

class EAN13Encoder:
    """Top-level class which handles the overall process of 
    encoding input number and outputting the result"""

    def __init__( self, code ):
        """Set up the encoder with the concatenated input values.
        code must be 12 digits long in the following format
        nnmmmmmppppp
        where n is the number system
        m is the manufacturing code
        p is the product code"""

        # Make sure it's 12 digits long
        if code.isdigit() and len(code) == 12:
            self.code = code
            self.check_digit = self.calculate_check_digit()
            self.full_code = self.code + str(self.check_digit)
            self.left_bars = ""
            self.right_bars = ""
            self.encode( )
        else:
            raise Exception("code must be 12 digits long")


    def encode( self ):
        """Encode the barcode number and return the left and right
        data strings"""

        parity_values = self.get_parity( )

        self.left_bars = ""
        self.right_bars = ""

        # Exclude the first number system digit, this was
        # for determining the left parity
        for parity, digit in zip( parity_values, self.full_code[1:7] ):
            self.left_bars += encoding.get_left_encoded( int(digit), parity ) 
        for digit in self.full_code[7:]:
            self.right_bars += encoding.get_right_encoded( int(digit) )

        return self.left_bars, self.right_bars


    def get_parity( self ):
        """Return the parity mappings applicable to this code"""
        return encoding.parity_table[int(self.code[0])]

    def calculate_check_digit( self ):
        """Modulo-10 calculation of the barcode check digit
        First, we take the rightmost digit of the value and consider it to be 
        an "odd" character. We then move right-to-left, alternating between 
        odd and even. We then sum the numeric value of all the even positions, 
        and sum the numeric value multiplied by three of all the 
        odd positions."""

        def sum_str( total, digit ):
            """add a stringified digit to the total sum"""
            return total + int(digit)

        # sum the "odd" digits (1,3,5,7,9,11) and multiply by 3
        oddsum = reduce( sum_str, self.code[1::2], 0 )

        # sum the "even" digits (0,2,4,6,8,10)
        evensum = reduce( sum_str, self.code[:12:2], 0 )

        # add them up
        total = oddsum*3 + evensum

        # check digit is the number that can be added to the total
        # to get to a multiple of 10
        return (10 - (total % 10)) % 10


    def save( self, filename, bar_width=3 ):
        """Write the barcode out to an image file"""
        EAN13Renderer( self.full_code, 
                        self.left_bars, 
                        self.right_bars, 
                        GUARDS ).write_file( filename, bar_width )



#!/usr/bin/env python
"""Coverage and unittest for datamatrix and QR Code library"""
__revision__ = "$Revision: 2025 $"

import sys, unittest
sys.path.append('.')
import coverage # get it from http://www.nedbatchelder.com/code/modules/coverage.html

coverage.erase()
coverage.start()

import qrcode.qrcodetest
suite = unittest.TestLoader().loadTestsFromName('qrcode.qrcodetest.MatrixTest')
unittest.TextTestRunner().run(suite)

import datamatrix.matrixtest
suite = unittest.TestLoader().loadTestsFromName('datamatrix.matrixtest.MatrixTest')
unittest.TextTestRunner().run(suite)

import ean13.eantest
suite = unittest.TestLoader().loadTestsFromName('ean13.eantest.EAN13Test')
unittest.TextTestRunner().run(suite)

import code128.code128test
suite = unittest.TestLoader().loadTestsFromName('code128.code128test.Code128Test')
unittest.TextTestRunner().run(suite)


coverage.stop()
coverage.report(['qrcode/__init__.py',
                 'qrcode/isodata.py',
                 'qrcode/qrcodetest.py',
                 'qrcode/renderer.py',
                 'qrcode/textencoder.py',
                 'datamatrix/__init__.py',
                 'datamatrix/placement.py',
                 'datamatrix/renderer.py',
                 'datamatrix/reedsolomon.py',
                 'datamatrix/textencoder.py',
				 'ean13/__init__.py',
				 'ean13/encoding.py',
				 'ean13/renderer.py',
				 'code128/__init__.py',
				 'code128/encoding.py',
				 'code128/textencoder.py',
				 'code128/renderer.py'
                 ])



"""Text encoder for QR Code encoder"""

__revision__ = "$Rev: 1927 $"

import logging

import isodata as isodata

LOG = logging.getLogger("qrcode")


class BitStream:
    """Simple Bit stream implementation"""

    def __init__(self):

        self.data = []
    # end def __init__


    def append(self, value, bitsnum):
        """Append 'bitsnum' bits to the end of bit stream"""

        if (bitsnum < 1):
            raise ValueError("Wrong value for number of bits (%d)" % bitsnum)
        # end if
        for i in range(bitsnum - 1, -1, -1):
            self.data.append((value >> i) & 0x01)
        # end for
    # end def append


    def prepend(self, value, bitsnum):
        """Prepend 'bitsnum' bits to the begining of bit stream"""

        if (bitsnum < 1):
            raise ValueError("Wrong value for number of bits (%d)" % bitsnum)
        # end if
        for i in range(0, bitsnum, 1):
            self.data.insert(0, (value >> i) & 0x01)
        # end for
    # end def prepend
# end class BitStream


class TextEncoder:
    """Text encoder class for QR Code"""

    def __init__(self):

        self.version = None
        self.ecl = None
        self.codewords = []
        self.matrix = None
        self.mtx_size = 0
        self.minfo = None
        self.max_data_codewords = None
    # end def __init__


    def encode(self, text, ecl=None):
        """Encode the given text and add padding and error codes
        also set up the correct matrix size for the resulting codewords"""

        self.__init__()
        if ecl is None:
            ecl = 'M'
        # end if
        str2ecl = {"L":1, "l":1, "M":0, "m":0, "Q":3, "q":3, "H":2, "h":2}
        self.ecl = str2ecl[ecl]

        self.encode_text(text)

        self.pad()

        self.minfo = isodata.MatrixInfo(self.version, self.ecl)

        self.append_error_codes()

        LOG.debug("Codewords: " +
                ' '.join([str(codeword) for codeword in self.codewords]))

        self.create_matrix()

        return self.matrix
    # end def encode


    def encode_text(self, text):
        """Encode the given text into bitstream"""

        char_count_num = 8
        result_len = 4 + 8 * len(text)
        terminator_len = 4
        # Calculate smallest symbol version
        for self.version in range(1, 42):
            if self.version == 10:
                char_count_num = 16
                result_len += 8
            elif self.version == 41:
                raise ValueError("QRCode cannot store %d bits" % result_len)
            # end if

            max_bits = isodata.MAX_DATA_BITS[self.version - 1 + 40 * self.ecl]
            if max_bits >= result_len:
                if max_bits - result_len < 4:
                    terminator_len = max_bits - result_len
                # end if
                self.max_data_codewords = max_bits >> 3
                break
            # end if
        # end for

        bitstream = BitStream()
        for char in text:
            bitstream.append(ord(char), 8)
        # end for

        bitstream.prepend(len(text), char_count_num)
        # write 'byte' mode
        bitstream.prepend(4, 4)
        # add terminator
        bitstream.append(0, terminator_len)
        # convert bitstream into codewords
        byte = 0
        bit_num = 7
        for bit in bitstream.data:
            byte |= bit << bit_num
            bit_num -= 1
            if bit_num == -1:
                self.codewords.append(byte)
                bit_num = 7
                byte = 0
            # end if
        # end for
    # end def encode_text


    def pad(self):
        """Pad out the encoded text to the correct word length"""

        pads = [236, 17]
        pad_idx = 0
        for _ in range(len(self.codewords), self.max_data_codewords):
            self.codewords.append(pads[pad_idx])
            pad_idx = 1 - pad_idx
        # end for
    # end def pad


    def append_error_codes(self):
        """Calculate the necessary number of error codes for the encoded
        text and padding codewords, and append to the codeword buffer"""

        i = 0
        j = 0
        rs_block_number = 0
        rs_temp = [[]]
        while i < self.max_data_codewords:

            rs_temp[rs_block_number].append(self.codewords[i])

            j += 1
            if (j >= self.minfo.rs_block_order[rs_block_number] -
                                                self.minfo.rs_ecc_codewords):
                j = 0
                rs_block_number += 1
                rs_temp.append([])
            # end if
            i += 1
        # end while

        rs_block_number = 0
        rs_block_order_num = len(self.minfo.rs_block_order)

        while rs_block_number < rs_block_order_num:
            rs_codewords = self.minfo.rs_block_order[rs_block_number]
            rs_data_codewords = rs_codewords - self.minfo.rs_ecc_codewords

            rstemp = rs_temp[rs_block_number]
            j = rs_data_codewords
            while j > 0:
                first = rstemp[0]
                if first != 0:
                    rstemp = rstemp[1:]
                    cal = self.minfo.rs_cal_table[first]

                    if (len(rstemp) < len(cal)):
                        rstemp, cal = cal, rstemp
                    # end if
                    cal += [0] * (len(rstemp) - len(cal))
                    rstemp = [x1 ^ x2 for x1, x2 in zip(rstemp, cal)]
                else:
                    rstemp = rstemp[1:]
                # end if
                j -= 1
            # end while
            self.codewords += rstemp
            rs_block_number += 1
        # end while
    # end def append_error_codes


    def create_matrix(self):
        """Create QR Code matrix"""

        matrix_content = self.minfo.create_matrix(self.version, self.codewords)
        self.mtx_size = len(matrix_content)

        LOG.debug("Matrix size is %d", self.mtx_size)

        mask_number = self.minfo.calc_mask_number(matrix_content)
        mask_content = 1 << mask_number

        format_info_value = ((self.ecl << 3) | mask_number)
        self.minfo.put_format_info(matrix_content, format_info_value)
        self.matrix = self.minfo.finalize(matrix_content, mask_content)
    # end def create_matrix
# end class TextEncoder




"""Unit test for code128 barcode encoder"""

__revision__ = "$Rev: 1$"

import unittest

from __init__ import Code128Encoder

class Code128Test(unittest.TestCase):
    """Unit test class for code128 bar code encoder"""

    test_strings = ("banana",
                     "wer das liest ist 31337",
                     "http://hudora.de/",
                     "http://hudora.de/artnr/12345/12/",
                     "http://hudora.de/track/00340059980000001319/",
                    )

    def test_charset_encoding( self ):
        """Make sure the character set encoding, code type switching
        and optimization works correctly"""
        known_good = \
        {    
            # dense C encoding
            # immediate switch into C-mode, should compress
            "1234": [105, 12, 34],

            # B only
            "hello": [104, 72, 69, 76, 76, 79],

            # B switching to C 
            "HI345678": [104, 40, 41, 99, 34, 56, 78], 

            # B => C => C, with leftover digit
            "HI34567A": [104, 40, 41, 99, 34, 56, 100, 23, 33],

            "BarCode 1": [104, 34, 65, 82, 35, 79, 68, 69, 0, 17]
        }
        for text, encoded in known_good.items():
            enc = Code128Encoder( text )
            self.assertEqual( enc.encoded_text, encoded )


    def test_check_sum( self ):
        """Make sure the checksum is calculated correctly"""
        
        known_good = \
        {
            "HI345678": 68,
            "BarCode 1": 33
        }

        for text, chk in known_good.items():
            enc = Code128Encoder( text )
            self.assertEqual( enc.checksum, chk )


    def test_bar_encoding( self ):
        """Make sure the bar encoding works correctly"""
        bars = "11010010000" + "11000101000" + "11000100010" + \
            "10111011110" + "10001011000" + "11100010110" + \
            "11000010100" + "10000100110" + "11000111010" + "11" 

        text = "HI345678"

        enc = Code128Encoder( text )
        self.assertEqual( enc.bars, bars )
                

    def test_against_generated(self):
        """Compare the output of this library with generated barcodes"""

        for index, string in enumerate(Code128Test.test_strings):
            encoder = Code128Encoder(string)
            encoder.save('test.png')

            import filecmp
            self.failUnless(filecmp.cmp('test.png',
                            'code128/test_img/%d.png' % (index + 1)))


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
"""Example code for qrcode library"""
__revision__ = "$Rev: 1927 $"

from qrcode import QRCodeEncoder
import sys
import logging

logging.getLogger("qrcode").setLevel(logging.DEBUG)
logging.getLogger("qrcode").addHandler(logging.StreamHandler(sys.stdout))

if __name__ == "__main__":
    ENCODER = QRCodeEncoder(sys.argv[1])
    ENCODER.save("test.png", 3)
    print ENCODER.get_ascii()
